/*
  Copyright (c) 2012 Vinícius dos Santos Oliveira

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in all
  copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  SOFTWARE.
  */

#ifndef TUFAO_ABSTRACTHTTPSERVERREQUESTHANDLER_H
#define TUFAO_ABSTRACTHTTPSERVERREQUESTHANDLER_H

#include "tufao_global.h"
#include <QtCore/QObject>
#include <QtCore/QStringList>

namespace Tufao {

class HttpServerRequest;
class HttpServerResponse;

/*!
  This class provides an interface for HttpServerRequest handlers.

  \since
  0.3
  */
class TUFAO_EXPORT AbstractHttpServerRequestHandler: public QObject
{
    Q_OBJECT
public:
    /*!
      Constructs an AbstractHttpServerRequestHandler object.
      */
    explicit AbstractHttpServerRequestHandler(QObject *parent = 0);

public slots:
    /*!
      Handles the \p request using the \p response object.

      \param args A list of options that may be used by the handler. The
      HttpServerRequestRouter class uses this argument to send the captured
      texts in the regex pattern.

      \retval true
      If the handler has responded to the request.

      \retval false
      If the requested page can't be generated by this handler. The connection
      should remain open and the \p response object shouldn't be used, leaving
      the response free to be used by other handlers in the chain.
      */
    virtual bool handleRequest(Tufao::HttpServerRequest *request,
                               Tufao::HttpServerResponse *response,
                               const QStringList &args = QStringList()) = 0;
};

} // namespace Tufao

#endif // TUFAO_ABSTRACTHTTPREQUESTHANDLER_H
